/* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */


using System;
using System.Collections.Generic;
using System.Text;
using Sagittarius;

namespace Sagittarius.Metadata
{
	public class KeywordManager
	{
		private Keywords[] keywordsArray;
		//private List<string> nonCommonKeywords, commonKeywords;
		private int numOfImages;
		
		public KeywordManager(string[] keywordsStrings)
		{
			//Console.WriteLine("Enering KeywordManager.Constructor(string[])");
			numOfImages = keywordsStrings.Length;
			keywordsArray = new Keywords[numOfImages];
			for (int i = 0; i<numOfImages;i++)
			{
				Keywords keywords = new Keywords(keywordsStrings[i]);
				keywordsArray[i]=keywords;
			}
			//Console.WriteLine("Leaving KeywordManager.Constructor(string[])");
		}
		
		public KeywordManager (Keywords[] keywordsArray)
		{
			//Console.WriteLine("Entering KeywordManager.Constructor(Keywords[])");
			numOfImages = keywordsArray.Length;
			this.keywordsArray=keywordsArray;
			//Console.WriteLine("Leaving KeywordManager.Constructor(Keywords[])");
		}
		
		public string GenerateUnion ()
		{
			//Console.WriteLine("Entering KeywordManager.GenerateUnion");
			List<string> commonKeywords = new List<string>();
			// Take the first image
			// as the common keywords are to present in every image, (that's why they are common)
			// if i examine the keywords of the first image, if they are presented in other images, i get the common keywords
			// iterating through the keywords of the first image
			//Console.WriteLine("Number of keywords in the first photo:"+keywordsArray[0].KeywordList.Count);
			if(keywordsArray[0].KeywordList.Count>0)
			{
				foreach(string kwOfFirstPhoto in keywordsArray[0].KeywordList)
				{
					//Console.WriteLine("exmining keywords. current keyword is: "+kwOfFirstPhoto);
					bool[] present = new bool[numOfImages]; 
					for (int j = 0; j<present.Length;j++)
					{ 
						//Console.WriteLine("Maybe here");
						present[j]=false; 
					}
					//Console.WriteLine("Looking up keyword: "+kwOfFirstPhoto);
					// finding matches in the images, starting with the second one
					for (int k = 1; k<numOfImages;k++)
					{
						//Console.WriteLine("Or here");
						//Console.WriteLine("Looking up in the "+k+" image");
						// every keyword shall be examined
						//Console.WriteLine("Checking if the keyword of the first photo: "+kwOfFirstPhoto+ " exist in the keywordlist of the "+(k+1)+" photo");
						if(keywordsArray[k].Contains(kwOfFirstPhoto))
						{
							present[k]=true;
							//Console.WriteLine("It is present");
						}
						/*foreach(string kwOfActualPhoto in individualKeywords[k].KeywordList)
						{
							//Console.WriteLine("The actual keyword of the "+k+"th image is: "+kwOfActualPhoto);
							if(kwOfFirstPhoto==kwOfActualPhoto)
							{
								//Console.WriteLine("It is a match!");
								present[k]=true; // if it is presented in an image, the corresponding bool value in the present array is to be set
							}
						}*/
					}
					bool common=true;
					//Console.WriteLine("Checking where it was found...");
					
					// checking, if the actual keyword of the first photo was found in every other photo or not
					for (int k = 1;k<numOfImages;k++)
					{
						common = common && present[k];
						//Console.WriteLine(common);
					}
					if(common) // if it was found, than we have a common keyword
					{
						//Console.WriteLine("The Keyword "+kwOfFirstPhoto+ " is a common one");
						commonKeywords.Add(kwOfFirstPhoto);
					}
				}
			}
			// getting noncommons
			List<string> nonCommonKeywords = new List<string>();
			for (int i = 0; i<numOfImages;i++) //iterating through the images
			{
				foreach(string actualKw in keywordsArray[i].KeywordList) // iterating through the keywords
				{
					bool presentInCommon=false;
					bool presentInNonCommon=false;
					// looking the actual keyword up in the noncommon list
					foreach(string nonCommonKw in nonCommonKeywords)
					{
						if(nonCommonKw==actualKw)
						{
							presentInNonCommon=true;
							break;
						}
					}
					// looking the actual keyword up in the common list
					foreach(string commonKw in commonKeywords)
					{
						if(commonKw==actualKw)
						{
							presentInCommon=true;
							break;
						}
					}
					if (!(presentInNonCommon||presentInCommon))
					{
						nonCommonKeywords.Add(actualKw);
					}
				}
			}
			/*Console.WriteLine("----------------");
			foreach(string s in nonCommonKeywords)
			{
				Console.WriteLine("Noncommon: "+s);
			}
			foreach(string s in commonKeywords)
			{
				Console.WriteLine("Common: "+s);
			}*/
			nonCommonKeywords.Sort();
			commonKeywords.Sort();
			
			StringBuilder sr = new StringBuilder();
			foreach(string kw in commonKeywords)
			{
				sr.Append(kw+", ");
			}
			foreach(string kw in nonCommonKeywords)
			{
				sr.Append("*"+kw+", ");
			}
			sr.Remove(sr.Length-2,2);
			//Console.WriteLine("Leaving KeywordManager.GenerateUnion");

			return sr.ToString();
		}
		
		public void SetKeywords(string keywordsString)
		{
			//Console.WriteLine("Entering KeywordManager.SetKeywords"); 
			//Console.WriteLine("InputKeywordlist is: "+keywordsString);

			string[] inputKeywords = keywordsString.Split(',');

			//Console.WriteLine("It contains "+inputKeywords.Length+" keywords");

			Keywords nonCommonKeywords = new Keywords();
			Keywords commonKeywords = new Keywords();
			Keywords allKeywords = new Keywords();

			for(int i=0; i<inputKeywords.Length;i++)
			{
				//Console.Write("Trimming keyword: '"+inputKeywords[i]+"'");
				inputKeywords[i] = inputKeywords[i].Trim(',','.',' ');
				//Console.WriteLine(" . Result after trim: '"+inputKeywords[i]+"'");
				if ((inputKeywords[i]=="")||(inputKeywords[i]=="*"))
				{
					//Console.WriteLine("Hey,I found an invalid keyword! it is '"+inputKeywords[i]+"'"); 
					continue;
				}
				if (inputKeywords[i].StartsWith("*")) // ha a keyword noncommon
				{
					nonCommonKeywords.AddKeyword(inputKeywords[i].TrimStart('*',' '));
					//Console.WriteLine("Keyword: "+inputKeywords[i]+ " is noncommon, adding to noncommonlist.");
				}
				else // ha a keyword common
				{
					commonKeywords.AddKeyword(inputKeywords[i]);
					//Console.WriteLine("Keyword: "+inputKeywords[i]+ " is common, adding to commonlist.");
				}
				allKeywords.AddKeyword(inputKeywords[i].TrimStart('*'));
			}
			// ami fixen benne van a listában, azoknál ellenőrizni kell, h megvannak-e
			// ha nincsenek, hozzá kell adni
			//Console.WriteLine("Checking common keywords");
			foreach (string commonKeyword in commonKeywords.KeywordList)
			{
				//Console.WriteLine("Checking if keyword: "+commonKeyword+" is present among every image's keywords");
				int k=1;
				foreach(Keywords keywords in keywordsArray) //végigiterálunk minden fájlon és megnézzük, van e olyan keywordje
				{
					if(keywords.Contains(commonKeyword))
					{
						//Console.WriteLine("Among the "+k+"nd image's keywords, it present, good.");
					}
					else
					{
						keywords.AddKeyword(commonKeyword);
						//Console.WriteLine("Among the "+k+"nd image's keywords, it is not present, adding.");
					}
					k++;
				}
			}
			//Console.WriteLine("Checking noncommon keywords.");
			// a csillagos kezdetüeket hagyni kell, ha benne vannak az adott kép keyword-listában
			// de azokat a keywordöket, amik nincsenek a csillagosok között de bennvannak a kép keywordlistájában, törölni kell
			int j=1;
			foreach(Keywords keywords in keywordsArray) //minden kép...
			{
				//Console.WriteLine("Checking image "+j);
				for(int i=0; i<keywords.KeywordList.Count;i++)  // minden keywordjén...
				{
					//Console.WriteLine("Checking keyword: "+keywords.KeywordList[i]);
					if(!allKeywords.Contains(keywords.KeywordList[i])) // ha a keyword nincs a bemeneti keywordok között
					{
						//Console.WriteLine("Keyword is not among noncommons or commons, removing");
						keywords.RemoveKeyword(keywords.KeywordList[i]);
						i--;
					}
					else
					{
						//Console.WriteLine("Keyword is among noncommons or commons, leaving it");
					}
				}
				j++;
			}
			foreach(Keywords keywords in keywordsArray) //minden kép...
			{
				keywords.KeywordList.Sort();
			}
			//Console.WriteLine("Entering KeywordManager.SetKeywords"); 
		}
	}
}
