/* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */


using System;
using System.Collections.Generic;
using System.Text;

namespace Sagittarius.Metadata
{
	public class Keywords
	{
		private List<string> keywordList;
		
		public Keywords(string keywordString)
		{
			Reset(keywordString);
		}
		
		public Keywords()
		{
			keywordList = new List<string>();
		}
		
		public void AddKeyword(string keyword)
		{
			string trimmedKeyword= keyword.Trim(new char[]{',',' ','.'});
			keywordList.Add(trimmedKeyword);
		}
		
		public void RemoveKeyword(string keyword)
		{
			keywordList.Remove(keyword);
		}
		
		public List<string> KeywordList
		{
			get { return keywordList; }
		}
		
		public bool Contains (string keywordToLook)
		{
			return keywordList.Contains(keywordToLook);
		}
		
		public override string ToString ()
		{
			StringBuilder sr = new StringBuilder();
			foreach(string kw in keywordList)
			{
				//sr.Append("*"+kw+", ");
				sr.Append(kw+", ");
			}
			try
			{
				sr.Remove(sr.Length-2,2);
			}
			catch
			{
			}
			return sr.ToString();
		}
		
		public void Reset (string keywordString)
		{
			keywordList = new List<string>();
			string[] keywords = keywordString.Split(',');
			
			foreach(string keyword in keywords)
			{
				AddKeyword(keyword);
			}
		}

		
	}
}
