/* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using Sagittarius.Exiftool;

namespace Sagittarius.Metadata
{
	public class IptcSection :MetadataSection
	{
		private Keywords keywords;
		
		public IptcSection():base()
		{
			//Console.WriteLine("Entering IptcSection constructor");
			this.SectionName=Strs.XMP_IPTC_SECT_NAME;
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_ADR_CITY,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_ADR_CTRY,"")); 
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_ADR_EXTADR,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_ADR_PCODE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_ADR_REGION,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_EMAIL_WORK,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_TEL_WORK,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CI_URL_WORK,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CITY,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_COPYRIGHT_NOTICE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_COUNTRY,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_COUNTRY_CODE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CREATOR,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_CREATOR_JOB_TITLE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_DATE_CREATED,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_DESCRIPTION,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_DESCRIPTION_WRITER,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_HEADLINE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_INSTRUCTIONS,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_INTELLECTUAL_GENRE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_JOB_ID,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_KEYWORDS,""));
			keywords = new Keywords();
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_LOCATION,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_PROVIDER,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_PROVINCE_STATE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_RIGHT_USAGE_TERMS,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_SCENE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_SOURCE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_SUBJECT_CODE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_TITLE,""));
			base.AddElement(new MetadataElement(ExiftoolTags.IPTC_COPYRIGHTED,""));
			//Console.WriteLine("Leaving IptcSection constructor");
		}
		
		public override void AddElement(MetadataElement element)
		{
			throw new Exception("IptcSection cannot have other elements than the predefined ones"); 
		}
		
		/*public override MetadataElement GetElement (string key)
		{
			if (key!=ExiftoolTags.IPTC_KEYWORDS)
			{
				return base.GetElement(key);
			}
			else
			{
				return new MetadataElement(ExiftoolTags.IPTC_KEYWORDS,keywords.ToString());
			}
		}*/
		
		public override void SetElement (string key, string newValue)
		{
			//Console.WriteLine("SetElement");
			if (key==ExiftoolTags.IPTC_KEYWORDS)
			{
				//Console.WriteLine("---------------------------");
				//Console.WriteLine("SetElement: ElementName: "+key+"=="+ExiftoolTags.IPTC_KEYWORDS);
				//Console.WriteLine(" Setting keywords to:"+ newValue);
				keywords.Reset(newValue);
				base.SetElement(key, newValue);
				//Console.WriteLine("The set keywords are: "+GetValue(key));
				//Console.WriteLine("---------------------------");
			}
			else
			{
				//Console.WriteLine("Normal element, key: "+key);
				base.SetElement(key, newValue);
			}
		}
		
		public Keywords Keywords 
		{
			get { return keywords; }
			set { keywords = value; }
		}
		
		public new IptcSection Copy()
		{
			//Console.WriteLine("Entering IptcSection.copy");
			IptcSection result = new IptcSection();
			foreach (MetadataElement element in elementsList)
			{
				result.SetElement(element.Key,element.Value);
			}
			//Console.WriteLine("Leaving IptcSection.copy");
			return result;
		}
		
		public void UpdateKeywords()
		{
			base.SetElement(ExiftoolTags.IPTC_KEYWORDS, keywords.ToString());
		}
	}
}
